package algorithm

const (
	// DigestSHA1 is te name for SHA1 digests.
	DigestSHA1 = "sha1"

	// DigestSHA224 is te name for SHA224 digests.
	DigestSHA224 = "sha224"

	// DigestSHA256 is te name for SHA256 digests.
	DigestSHA256 = "sha256"

	// DigestSHA384 is te name for SHA384 digests.
	DigestSHA384 = "sha384"

	// DigestSHA512 is te name for SHA512 digests.
	DigestSHA512 = "sha512"
)

const (
	// SaltLengthDefault is the default salt size for most implementations.
	SaltLengthDefault = 16

	// KeyLengthDefault is the default key size for most implementations.
	KeyLengthDefault = 32
)
