package plaintext

const (
	// EncodingFmt is the encoding format for this algorithm.
	EncodingFmt = "$%s$%s"

	// AlgName is the name for this algorithm.
	AlgName = "plaintext"

	// AlgIdentifierPlainText is the identifier used in encoded plaintext variants of this algorithm.
	AlgIdentifierPlainText = AlgName

	// AlgIdentifierBase64 is the identifier used in encoded base64 variants of this algorithm.
	AlgIdentifierBase64 = "base64"
)
