// Package sha1crypt provides helpful abstractions for an implementation of crypt (SHA1) and implements
// github.com/go-crypt/crypt interfaces.
//
// This implementation is loaded by crypt.NewDecoderAll.
package sha1crypt
