// Package plaintext implements github.com/go-crypt/crypt interfaces with variants of plaintext useful for easy uptake
// of previously unhashed passwords.
//
// This implementation is loaded by crypt.NewDecoderAll.
package plaintext
