// Package shacrypt provides helpful abstractions for an implementation of SHA-crypt and implements
// github.com/go-crypt/crypt interfaces.
//
// See https://www.akkadia.org/drepper/SHA-crypt.html for specification details.
//
// This implementation is loaded by crypt.NewDefaultDecoder and crypt.NewDecoderAll.
package shacrypt
