// Package pbkdf2 provides helpful abstractions for an implementation of PBKDF2 and implements
// github.com/go-crypt/crypt interfaces.
//
// This implementation is loaded by crypt.NewDefaultDecoder and crypt.NewDecoderAll.
package pbkdf2
