// Package scrypt provides helpful abstractions for an implementation of RFC7914 and implements
// github.com/go-crypt/crypt interfaces.
//
// This implementation is loaded by crypt.NewDefaultDecoder and crypt.NewDecoderAll.
package scrypt
