// Package md5crypt provides helpful abstractions for an implementation of crypt (MD5) and implements
// github.com/go-crypt/crypt interfaces.
//
// This implementation is loaded by crypt.NewDecoderAll.
package md5crypt
